using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using Dabrorius.MonoPunk;

namespace Seminar
{

	public class Eater : Entity 
	{
		protected int plantsEaten;
		protected int direction;
		protected int state;
		private int starveCounter;
		private int notAliveCounter;
		private Spritemap spritemap;
		private bool alive;
		public Eater(float x = 0, float y = 0) : base(x,y)
		{
			state = 0;
			direction  = MP.Rand(360/Config.ANGLE_STEP) * Config.ANGLE_STEP;
			plantsEaten = 0;
			
			spritemap = new Spritemap(Engine.EmbeddFile("monster.png"),24,24);
		
			spritemap.Add("down",new int[]{0});
			spritemap.Add("up",new int[]{1});
			spritemap.Add("right",new int[]{2});
			spritemap.Add("left",new int[]{3});
			spritemap.Add("dead",new int[]{4});

			setAnimation();
			
			notAliveCounter = 0;
			alive = true;
			Type = "eater";
			Graphic = spritemap;
			setHitbox(16,16);
			turnRight();
			
		}
		
		private void setAnimation()
		{
			if( direction == 180 ) spritemap.Play("up");
			if( direction == 270 ) spritemap.Play("left");
			if( direction == 0 ) spritemap.Play("down");
			if( direction == 90 ) spritemap.Play("right");

		}
		
		virtual public Eater Clone()
		{
			return new Eater(0,0);
		}
				
		public int Fitness 
		{
			get { return (plantsEaten + 1);}
			
		}
		
		virtual public Eater mate(Eater other)
		{
			return null;
		}
		
		virtual public int getGene(int position)
		{
			return 0;
		}
		
		virtual public void setGene(int position, int newValue)
		{
			
		}
		
		virtual public void randomGenotype()
		{
			
		}
				
		protected int typeInFront
		{
			get 
			{ 
				if ( forwardPositionSight.X >= EaterWorld.worldSize.X ||
				    forwardPositionSight.Y >= EaterWorld.worldSize.Y ||
				    forwardPositionSight.X < 0 || forwardPositionSight.Y < 0)
					return 3;
				if (Collide("eater",forwardPositionSight.X, forwardPositionSight.Y) != null ) return 2;
				if (Collide("plant",forwardPositionSight.X, forwardPositionSight.Y) != null ) return 1;
				else return 0;
			}
		}
		
		protected int typeBehind
		{
			get 
			{ 
				if ( backPositionSight.X >= EaterWorld.worldSize.X ||
				    backPositionSight.Y >= EaterWorld.worldSize.Y ||
				    backPositionSight.X < 0 || backPositionSight.Y < 0)
					return 3;
				// if other eater return 2
				if (Collide("eater",backPositionSight.X, backPositionSight.Y) != null ) return 2;
				if (Collide("plant",backPositionSight.X, backPositionSight.Y) != null ) return 1;
				else return 0;
			}
		}
		
		
		
		protected Vector2 forwardPosition
		{
			get
			{
				return relativePosition(0, Config.STEP_SIZE);
				/*
				double xStep = Math.Cos( MP.Degs2Rad(direction)) * Config.CELL_SIZE;
				double yStep = Math.Sin(MP.Degs2Rad(direction)) * Config.CELL_SIZE;
				return new Vector2(X + (float) xStep, Y + (float) yStep);
				*/
			}
		}
		
		protected Vector2 forwardPositionSight
		{
			get
			{
				return relativePosition(0, Config.CELL_SIZE);
			}
		}
		
		public void setRandomPosition()
		{
			X = Config.CELL_SIZE * MP.Rand(Config.GRID_WIDTH);
			Y = Config.CELL_SIZE * MP.Rand(Config.GRID_HEIGHT);
		}
		
		override public void Added()
		{
			plantsEaten = 0;
			starveCounter = 0;
		}
		
		protected Vector2 backPosition
		{
			get
			{
				return relativePosition(0, -Config.STEP_SIZE);
				/*
				float xStep = (float) Math.Cos(MP.Degs2Rad(direction+180)) * Config.CELL_SIZE;
				float yStep = (float) Math.Sin(MP.Degs2Rad(direction+180)) * Config.CELL_SIZE;
				return new Vector2(X + xStep, Y + yStep);
				*/
			}
		}
		
		protected Vector2 backPositionSight
		{
			get
			{
				return relativePosition(0, -Config.CELL_SIZE);
				/*
				float xStep = (float) Math.Cos(MP.Degs2Rad(direction+180)) * Config.CELL_SIZE;
				float yStep = (float) Math.Sin(MP.Degs2Rad(direction+180)) * Config.CELL_SIZE;
				return new Vector2(X + xStep, Y + yStep);
				*/
			}
		}
		
		
		public void resetFitness()
		{
			plantsEaten = 0;
		}
		
		override public void Update()
		{
			if( ! alive ) 
			{
				notAliveCounter++;
				
				if( notAliveCounter > 8) MP.CurrentWorld.Remove(this);
			}	
			else
			{		
				starveCounter ++;
				
				this.Act();

				Plant p = Collide("plant",X,Y) as Plant;
				if( p != null )
				{
					p.Eat();
					plantsEaten++;
					starveCounter = 0;
				}
				
				if( starveCounter > Config.TURNS_TO_STARVE && Config.TURNS_TO_STARVE > 0 )
				{
					alive = false;
					spritemap.Play("dead");
					//MP.CurrentWorld.Remove(this);
				}
			}
		}
		
		// PRIVATE
		virtual protected void Act()
		{
			// TO-OVERRIDE
		}
		
		protected void turnRight()
		{
			direction = (direction + Config.ANGLE_STEP) % 360;
			setAnimation();
			
		}
		
		protected void turnLeft()
		{
			direction = (direction - Config.ANGLE_STEP) % 360;
			if (direction < 0 ) direction += 360;
			setAnimation();
		}
		
		protected void moveForward()
		{
			if( typeInFront < 2 )
			{
				X = forwardPosition.X;
				Y = forwardPosition.Y;
			}
		}
		
		protected void moveBackward()
		{
			if( typeBehind < 2 )
			{
				X = backPosition.X;
				Y = backPosition.Y;
			}
		}
		
		protected Vector2 relativePosition(Vector2 point)
		{
			float cosDirection = (float) Math.Cos( MP.Degs2Rad(direction) ); 
			float sinDirection = (float) Math.Sin( MP.Degs2Rad(direction) ); 
			float xPos = point.X * cosDirection - point.Y * sinDirection;
			float yPos = point.X * sinDirection + point.Y * cosDirection;
			return new Vector2(X + xPos,Y + yPos);
		}
		

		
		protected Vector2 relativePosition(float xPos, float yPos)
		{
			return relativePosition(new Vector2(xPos,yPos) );
		}
	}
}

