using System;
using Dabrorius.MonoPunk;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

using System.Collections.Generic;
namespace Seminar
{
	public class EaterWorld : World
	{
		public double stepDuration = 0.1;
		public double stepCounter = 0;
		
		private int generationPointer = 0;

		
		int numberOfTurns = 0;
		Generation generation;
		
		public EaterWorld ()
		{
			generation = new Generation(Config.POPULATION_SIZE);
		}
		
		override public void Begin()
		{
			Backdrop bg = new Backdrop(Engine.EmbeddFile("bg.png"));
			AddGraphic(bg);
			generation.generateInitial(Config.PROTOTYPE);
			addNextBatch();
		}
		
		private void reGeneratePlants()
		{	
			Backdrop bg = new Backdrop(Engine.EmbeddFile("bg.png"));
			AddGraphic(bg);
			
			for( int b = 0; b < Config.BIG_CLUMPS; b++)
			{
				addPlantsClump(4);
			}
			
			for( int m = 0; m < Config.MEDIUM_CLUMPS; m++)
			{
				addPlantsClump(3);
			}
			
			for( int b = 0; b < Config.SMALL_CLUMPS; b++)
			{
				addPlantsClump(2);
			}
		}
		

		
		private void addNextBatch()
		{
			
			
			
			if( generationPointer >= Config.POPULATION_SIZE )
			{
				generationPointer = 0;
				generateNewGeneration(Config.POPULATION_SIZE);
				return;
			}
			
			RemoveAll();
			reGeneratePlants();
					
			for( int i = 0; i < Config.ENTITIES_PER_WORLD; i++ )
			{
				int toAdd = generationPointer + i;
				if( toAdd < Config.POPULATION_SIZE )
				{
					if( generation.generation[toAdd].Fitness == 1 )
					{
						Add (generation.generation[toAdd]);
					}
				}
			}
			
			generationPointer += Config.ENTITIES_PER_WORLD;
		}
		
		
		private void generateNewGeneration(int numberOfChildren)
		{
				RemoveAll();
			this.generation.newGeneration(Config.POPULATION_SIZE);
			addNextBatch();
		}
		
		private void addPlantsClump(int size)
		{
			int xPos = MP.Rand(Config.GRID_WIDTH - size + 1);
			int yPos = MP.Rand(Config.GRID_HEIGHT - size + 1);
				
			for( int i = 0; i < size; i++ )
			{
				for( int j=0;j < size; j++)
				{
					Plant p = new Plant( (xPos + i) * Config.CELL_SIZE , (yPos + j) * Config.CELL_SIZE);
	                Add(p);
				}
			}	
		}
		
		override public void Update()
		{
			checkWorldSpeedChange();	                
			stepCounter += MP.Elapsed;
			if( stepCounter > stepDuration )
			{
				numberOfTurns ++;
				stepCounter = 0;
				
				bool maxTurnsReached;
				if( Config.MAX_TURNS == 0 ) maxTurnsReached = false;
				else maxTurnsReached = (numberOfTurns >= Config.MAX_TURNS);
				
				bool allEatersStarved = (TypeCount("eater") == 0);
				
				if( maxTurnsReached || allEatersStarved)
				{
					numberOfTurns = 0;
					addNextBatch();
				}
				
				base.Update();

			}
		}
	
		public static Vector2 worldSize
		{
			get {return new Vector2(Config.GRID_WIDTH * Config.CELL_SIZE, 
				                    Config.GRID_HEIGHT * Config.CELL_SIZE);}
		}
		
		private void checkWorldSpeedChange()
		{			
			if( Input.Pressed(Keys.Up) )
			{
				if( stepDuration > 0.01 )
				{
					System.Console.WriteLine("Increasing speed +");
					stepDuration /= 2;
				}
			}
			
			if( Input.Pressed(Keys.Down) )
			{
				System.Console.WriteLine("Decreasing speed -");
				stepDuration *= 2;
			}	
		}
	}
}

